unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, PDF, PtblRV, RVReport, RVStyle, ShellApi, ExtCtrls;

type
  TForm1 = class(TForm)
    pdf: TPDFDocument;
    btnSave: TButton;
    rvs: TRVStyle;
    rvrh: TRVReportHelper;
    btnOpen: TButton;
    od: TOpenDialog;
    sd: TSaveDialog;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    cmbPageSize: TComboBox;
    rgPageOrientation: TRadioGroup;
    GroupBox2: TGroupBox;
    Label2: TLabel;
    txtLeft: TEdit;
    Label3: TLabel;
    txtTop: TEdit;
    Label4: TLabel;
    txtRight: TEdit;
    Label5: TLabel;
    txtBottom: TEdit;
    lblStatus: TLabel;
    Image1: TImage;
    Image2: TImage;
    procedure btnSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure Image1Click(Sender: TObject);
    procedure Image2Click(Sender: TObject);
  private
    { Private declarations }
    function GetMargin(edit: TEdit): Integer;
    function GetPageSize: TPDFPageSize;
    function GetPageOrientation: TPDFPageOrientation;
    procedure ShowText(s: String; RedrawForm: Boolean);
    procedure ShowError(s: String);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
{------------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
begin
  rvrh.RichView.Style := rvs;
  rvrh.RichView.RTFReadProperties.ParaStyleMode := rvrsAddIfNeeded;
  rvrh.RichView.RTFReadProperties.TextStyleMode := rvrsAddIfNeeded;
  rvrh.RichView.RTFReadProperties.UnicodeMode := rvruOnlyUnicode;
  rvrh.MetafileCompatibility := True;

  cmbPageSize.ItemIndex := 1;
end;

procedure TForm1.btnOpenClick(Sender: TObject);
begin
  rvrh.RichView.Clear;
  rvrh.RichView.DeleteUnusedStyles(True,True,True);
  btnSave.Enabled := False;
  if od.Execute then
  begin
    Screen.Cursor := crHourGlass;
    ShowText('Loading RTF file...', True);
    if rvrh.RichView.LoadRTF(od.FileName) then
      btnSave.Enabled := True
    else
      ShowError('Load Error');
    ShowText('', False);
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.btnSaveClick(Sender: TObject);
var i, LeftMargin, TopMargin, RightMargin, BottomMargin, DocWidth, DocHeight: Integer;
  pt: TPoint;
const MinDocSize = 50;
begin
  if not sd.Execute then
    exit;

  LeftMargin   := GetMargin(txtLeft);
  TopMargin    := GetMargin(txtTop);
  RightMargin  := GetMargin(txtRight);
  BottomMargin := GetMargin(txtBottom);

  pdf.Resolution := Screen.PixelsPerInch;
  pdf.FileName := sd.FileName;
  pdf.BeginDoc;

  pdf.Page[0].Size := GetPageSize;
  pdf.Page[0].Orientation := GetPageOrientation;

  DocWidth  := pdf.Page[0].Width - (LeftMargin+RightMargin);
  DocHeight := pdf.Page[0].Height - (TopMargin+BottomMargin);

  if (DocWidth<MinDocSize) then
  begin
    ShowError('Horizontal margins are too large');
    txtLeft.SetFocus;
    pdf.Abort;
    exit;
  end;
  if (DocHeight<MinDocSize) then
  begin
    ShowError('Vertical margins are too large');
    txtTop.SetFocus;
    pdf.Abort;
    exit;
  end;

  Screen.Cursor := crHourGlass;
  try
    ShowText('Formatting...', True);
    rvrh.Init(pdf.Canvas, DocWidth);
    while rvrh.FormatNextPage(DocHeight) do;
    for i := 0 to rvrh.PagesCount-1 do
    begin
      ShowText(Format('Collecting PDF pages (%d of %d)... ', [i, rvrh.PagesCount]), False);
      if i>0 then
      begin
        pdf.NewPage;
        pdf.Page[i].Size        := pdf.Page[0].Size;
        pdf.Page[i].Orientation := pdf.Page[0].Orientation;
      end;
      // 3 lines below are used instead of
      // rvrh.DrawPageAt(LeftMargin, TopMargin, i+1, pdf.Canvas, False, DocHeight);
      // In newer version of LLPDFLib DrawPageAt must work too
      SetViewportOrgEx(pdf.Canvas.Handle, LeftMargin, TopMargin, @pt);
      rvrh.DrawPage(i+1, pdf.Canvas, False, DocHeight);
      SetViewportOrgEx(pdf.Canvas.Handle, pt.x, pt.y, nil);      
    end;
    ShowText('Writing PDF file...', True);
    pdf.EndDoc;
  finally
    Screen.Cursor := crDefault;
    ShowText('', False);
  end;
end;

{------------------------------------------------------------------------------}
// Returns margin, pixels
function TForm1.GetMargin(edit: TEdit): Integer;
var mm: Integer;
begin
  mm := StrToIntDef(edit.Text, 0);
  edit.Text := IntToStr(mm);
  Result := MulDiv(mm, 5*Screen.PixelsPerInch, 127);
end;
{------------------------------------------------------------------------------}
function TForm1.GetPageOrientation: TPDFPageOrientation;
begin
  Result := TPDFPageOrientation(rgPageOrientation.ItemIndex);
end;
{------------------------------------------------------------------------------}
function TForm1.GetPageSize: TPDFPageSize;
begin
  Result := TPDFPageSize(cmbPageSize.ItemIndex+1);
end;
{------------------------------------------------------------------------------}
procedure TForm1.ShowText(s: String; RedrawForm: Boolean);
begin
  if s='' then
    s := 'Ready';
  lblStatus.Caption := s;
  if RedrawForm then
    Update
  else
    lblStatus.Update;
end;
{------------------------------------------------------------------------------}
procedure TForm1.ShowError(s: String);
begin
  Application.MessageBox(PChar(s), 'Error', MB_OK or MB_ICONSTOP);
end;
{------------------------------------------------------------------------------}
procedure TForm1.Image1Click(Sender: TObject);
begin
  ShellExecute(0, 'open', 'http://www.llion.net/?From=RTF2PDF', nil, nil, SW_SHOW);
end;

procedure TForm1.Image2Click(Sender: TObject);
begin
  ShellExecute(0, 'open', 'http://www.trichview.com/?From=RTF2PDF', nil, nil, SW_SHOW);
end;


end.

